const { app, BrowserWindow, ipcMain, dialog } = require("electron");
const shell = require("electron").shell;
const getVlcСommand = require("vlc-command");

if (require("electron-squirrel-startup")) return app.quit();

const path = require("path");
const fs = require("fs");
const jf = require("jsonfile");
const WebSocket = require("ws");

const VLC = require("./vlc-simple-player");

let ws;
let userFile = null;
let mainWindow;
let startTime;

let player;

let lastHeartbeat = 0;
let connectionRequested = false;
let bookmarkRequestInProgress = false;

const config = getConfig();

async function handleFileOpen() {
  const { canceled, filePaths } = await dialog.showOpenDialog();
  if (canceled) {
    return null;
  } else {
    userFile = filePaths[0];
    return filePaths[0];
  }
}

const createWindow = () => {
  mainWindow = new BrowserWindow({
    autoHideMenuBar: true,
    width: 900,
    height: 740,
    icon: __dirname + "/favicon.ico",
    webPreferences: {
      preload: path.join(__dirname, "preload.js"),
    },
  });

  // mainWindow.setMenu(null);
  mainWindow.loadFile("index.html");
  mainWindow.webContents.once("dom-ready", () => {
    const config = getConfig();
    mainWindow.webContents.send(
      "report-saved-token",
      config.connectionToken ?? ""
    );

    if (config.history && config.history.length > 0) {
      userFile = config.history[0].filePath;
    }
    mainWindow.webContents.send(
      "history",
      config.history ? config.history : []
    );

    getVlcСommand((err, vlcPath) => {
      if (err) {
        mainWindow.webContents.send("vlc-check", "");
      } else {
        mainWindow.webContents.send("vlc-check", vlcPath);
      }
    });
  });
};

app.whenReady().then(() => {
  let playerPosition;

  if (config.autostart === "yes") {
    setupWebSockets(config.connectionToken);
  }

  ipcMain.handle("dialog:openFile", handleFileOpen);

  ipcMain.handle("useToken", async (event, token) => {
    config.connectionToken = token;
    saveConfig(config);
    setupWebSockets(token);
  });

  ipcMain.handle("disconnect", async () => {
    connectionRequested = false;
    lastHeartbeat = 0;

    ws.close();
    const conf = getConfig();
    conf.autostart = undefined;
    saveConfig(conf);
    mainWindow.webContents.send("report-disconnect");
  });

  ipcMain.handle("setWindowSize", async (event, size) => {
    mainWindow.setSize(size.width, size.height);
  });

  ipcMain.handle("vlc:go", () => {
    if (!fs.existsSync(userFile)) {
      mainWindow.webContents.send("bad-media-file", userFile);
    } else {
      player = new VLC(userFile);

      const savedPosition = getSavedPosition();
      if (savedPosition) {
        setTimeout(() => {
          player.request(
            `/requests/status.json?command=seek&val=${savedPosition}`,
            (_) => {
              // console.log(_);
            }
          );
        }, 1000);
      }

      addFileToHistory(userFile);

      startTime = new Date().getTime();

      player.on("statuschange", (error, status) => {
        if (status) {
          const upTime = new Date().getTime() - startTime;
          if (upTime > 1000 && status.state === "stopped") {
            player.quit();
          } else {
            if (ws && ws.readyState === 1) {
              let bookmarkProcessed = undefined;
              if (bookmarkRequestInProgress) {
                bookmarkRequestInProgress = false;
                bookmarkProcessed = true;
              }

              ws.send(
                JSON.stringify({
                  msgType: "position",
                  position: status.time,
                  bookmarkProcessed,
                })
              );
            }

            savePlaybackPosition(status.time);

            if (mainWindow && mainWindow.webContents) {
              playerPosition = status.time;
              mainWindow.webContents.send(
                "update-status",
                JSON.stringify({
                  time: status.time,
                  state: status.state,
                })
              );
            }
          }
        }
      });
    }
  });

  ipcMain.handle("vlc:pause", () => {
    player.request("/requests/status.json?command=pl_pause", () => {});
  });

  ipcMain.handle("vlc:play", () => {
    player.request("/requests/status.json?command=pl_play", () => {});
  });

  ipcMain.handle("setCurrentFile", (event, filePath) => {
    userFile = filePath;
  });

  ipcMain.handle("quit", (event, filePath) => {
    shell.openExternal("https://www.videolan.org/vlc/");
    setTimeout(() => {
      app.quit();
    }, 1000);
  });

  ipcMain.handle("vlc:movePosition", (event, delta) => {
    const newPosition = playerPosition + delta;
    player.request(
      "/requests/status.json?command=seek&val=" + newPosition,
      () => {
        // console.log("moved to", newPosition);
      }
    );
  });

  ipcMain.handle("openURL", (event, url) => {
    setTimeout(() => {
      shell.openExternal(url);
    }, 2000);
  });

  createWindow();

  app.on("activate", () => {
    if (BrowserWindow.getAllWindows().length === 0) createWindow();
  });
});

app.on("window-all-closed", () => {
  if (player) {
    player.quit();
  }
  app.quit();
});

function setupWebSockets(token) {
  connectionRequested = true;

  ws = new WebSocket("wss://subtitles-robot.com:3333");
  ws.onopen = () => {
    ws.send(
      JSON.stringify({
        msgType: "token",
        token,
        sender: "controller",
      })
    );
  };

  ws.onmessage = (event) => {
    const message = JSON.parse(event.data);
    if (message.msgType === "tokenOK") {
      // console.log("connection to server established [" + message.email + "]");
      sendHeartBeat();
      updateConnectionStatus();

      const conf = getConfig();
      conf.autostart = "yes";
      saveConfig(conf);

      mainWindow.webContents.send("report-connection", {
        color: "black",
        text: "waiting for incoming connection: " + message.email + "....",
      });
    } else if (message.msgType === "tokenError") {
      if (message.error === "tokenReuse") {
        mainWindow.webContents.send("report-connection", {
          color: "red",
          text: "cannot connect: password is being used on another controller at this moment",
          code: "showHelp",
        });
      } else if (message.error === "invalidToken") {
        mainWindow.webContents.send("report-connection", {
          color: "red",
          text: "cannot connect: invalid or expired password",
          code: "showHelp",
        });
      } else {
        mainWindow.webContents.send("report-connection", {
          color: "red",
          text: "cannot connect: unknown error",
        });
      }
    } else if (message.msgType === "jump") {
      player.request(
        "/requests/status.json?command=seek&val=" +
          Math.floor(message.position + 1),
        () => {
          if (message.bookmark) {
            bookmarkRequestInProgress = true;
          }
          // console.log("jump", message.position);
        }
      );
    } else if (message.msgType === "heartbeat") {
      lastHeartbeat = message.value;
    }
  };

  ws.onerror = (error) => {
    mainWindow.webContents.send("report-connection", {
      color: "red",
      text:
        "server or network error" +
        (error.message ? " (" + error.message + ")" : ""),
    });
  };

  ws.onclose = () => {
    console.log("connection closed.");
  };

  const sendHeartBeat = () => {
    setTimeout(() => {
      if (ws && ws.readyState === 1) {
        ws.send(
          JSON.stringify({
            msgType: "heartbeat",
            value: new Date().getTime(),
          })
        );

        sendHeartBeat();
      }
    }, 1000);
  };
}

function updateConnectionStatus() {
  if (connectionRequested) {
    const delta = new Date().getTime() - lastHeartbeat;
    if (lastHeartbeat !== 0 && delta < 3000) {
      mainWindow.webContents.send("report-connection", {
        color: "green",
        text: "webpage connected, latency: " + delta + "ms",
      });
      setTimeout(updateConnectionStatus, 1000);
    } else if (lastHeartbeat !== 0) {
      mainWindow.webContents.send("report-connection", {
        color: "red",
        text: "lost connection to webpage",
      });
      lastHeartbeat = 0;
      ws.close();
      connectionRequested = false;
    } else {
      setTimeout(updateConnectionStatus, 1000);
    }
  }
}

function addFileToHistory(filePath) {
  const config = getConfig();
  const history = config.history || [];
  const index = history.findIndex((item) => {
    return item.filePath === filePath;
  });

  if (index !== -1) {
    history.splice(index, 1);
  }
  history.unshift({ filePath });

  config.history = history;
  saveConfig(config);
}

function savePlaybackPosition(position) {
  const config = getConfig();
  const history = config.history || [];

  const index = history.findIndex((item) => {
    return item.filePath === userFile;
  });

  if (index !== -1) {
    history[index].position = position;
  } else {
    console.log("could not save playback position");
    history.unshift({ filename: userFile, position });
  }

  config.history = history;
  saveConfig(config);
}

function getSavedPosition() {
  const config = getConfig();
  const history = config.history || [];

  const index = history.findIndex((item) => {
    return item.filePath === userFile;
  });

  if (index !== -1 && history[index].position) {
    return history[index].position.toString();
  } else {
    return "";
  }
}

function getConfig() {
  const configFileName = path.join(app.getPath("userData"), "config.json");

  if (!fs.existsSync(configFileName)) {
    jf.writeFileSync(configFileName, {}, { spaces: 2 });
  }

  const content = fs.readFileSync(configFileName, "utf8");
  try {
    return JSON.parse(content);
  } catch (e) {
    jf.writeFileSync(configFileName, {}, { spaces: 2 });
    return {};
  }
}

function saveConfig(config) {
  const configFileName = path.join(app.getPath("userData"), "config.json");
  jf.writeFileSync(configFileName, config, { spaces: 2 });
}
